<?php
require_once ASD_APPLICATION_PATH . '/forms/AdminAbstract.php';

class Asd_Form_Options extends Asd_Form_AdminAnstract
{
    protected $_helper;

    protected $_wordingFields = array();

    protected $_markets;

    protected $_marketFields = array(
        'category',
        'subcategory',
    );

    protected $_credentialFields = array(
        'access_key',
        'secret_access_key',
    );

    public function init()
    {
        $this->_helper = Asd_Application::getModel('Amazon_FormHelper')
            ->setForm($this);

        $this->setDisableTranslator(TRUE);

        $this->addElement('text', 'access_key', array(
            'required' => TRUE,
            'filters' => array(
                'StripNewlines',
                'StringTrim',
            ),
            'label' => __('AWS Access Key ID', 'amazon-bar-plugin'),
        ));

        $this->addElement('password', 'secret_access_key', array(
            'required' => TRUE,
            'filters' => array(
                'StripNewlines',
                'StringTrim',
            ),
            'label' => __('AWS Secret Access Key', 'amazon-bar-plugin'),
        ));

        $this->addElement('text', 'tracking_id', array(
            'filters' => array(
                'StripNewlines',
                'StringTrim',
            ),
            'label' => __('Tracking ID', 'amazon-bar-plugin'),
        ));

        //
        // Wording
        //

        $defaults = Asd_Application::getConfig()->optionDefaults;
        foreach (Asd_Application::getConfig()->wording as $name=>$info) {
            $info = $info->toArray();
            $label = isset($info['label'])
                ? $info['label']
                : (string)$defaults->$name;
            $this->_wordingFields[$name] = $info;

            $this->addElement('text', $name, array(
                'filters' => array(
                    'StripNewlines',
                    'StringTrim',
                ),
                'label' => $label,
                'help' => FALSE,
            ));
        }

        //
        // Market fields
        //
        $this->_markets = Asd_Application::getModel('Amazon_Market')->getList();
        foreach ($this->_markets as $marketCode=>$marketLabel)
        {
            $marketLabel = (string)$marketLabel;

            $this->addElement('text', "{$marketCode}_affiliate_id", array(
                'filters' => array(
                    'StripNewlines',
                    'StringTrim',
                ),
                'validators' => array(
                    array('Regex', TRUE,
                        array(
                            'pattern' => '~^[a-zA-Z0-9\-\.]+$~u',
                            'messages' => array(
                                'regexNotMatch' => __("'%value%' contains invalid characters", 'amazon-bar-plugin'),
                            ),
                        ),
                    )
                ),
                'label' => sprintf(__('%s Associate ID', 'amazon-bar-plugin'), $marketLabel),
                'help' => FALSE,
            ));

            $this->addElement('select', "{$marketCode}_category", array(
                'filters' => array(
                    'Null',
                ),
                'label' => sprintf(__('%s Category', 'amazon-bar-plugin'), $marketLabel),
                'disabled' => TRUE,
                'help' => FALSE,
            ));

            $this->addElement('select', "{$marketCode}_subcategory", array(
                'filters' => array(
                    'Null',
                ),
                'label' => sprintf(__('%s Subcategory', 'amazon-bar-plugin'), $marketLabel),
                'disabled' => TRUE,
                'help' => FALSE,
            ));
        }
    }

    /**
     * Load categories && subcategories list
     */
    public function loadCategories($data=NULL)
    {
        $this->_helper->loadCategories($data);

        return $this;
    }

    /**
     * Check data
     */
    public function isValidEx($data)
    {
        $result = $this->_helper->isValid($data);

        if ($result) {
            // Check affiliate_id for default market
            $defaultMarket = Asd_Application::getModel('Amazon_Market')->suggest();
            $elem = $this->getElement("{$defaultMarket}_affiliate_id");
            if ('' == $elem->getValue()) {
                $elem->addError(__('Value is required and cannot be empty', 'amazon-bar-plugin'));
                $result = FALSE;
            }
        }

        return $result;
    }

    public function extractCredentials($data=NULL)
    {
        $result = array();

        foreach ($this->_credentialFields as $field) {
            $value = $this->_extractField($field, $data);
            if ('' == $value) {
                return NULL;
            }
            $result[$field] = $value;
        }

        return $result;
    }

    protected function _extractField($field, $data=NULL)
    {
        if (is_null($data)) {
            return $this->getValue($field);
        }
        if (isset($data[$field])) {
            return $data[$field];
        }

        return NULL;
    }

    public function prepare()
    {
        foreach (array_keys($this->_markets) as $marketCode) {
            if (!$this->_hasCredentials($marketCode)) {
                foreach ($this->_marketFields as $field) {
                    $this->getElement("{$marketCode}_{$field}")->setAttrib('disabled', TRUE);
                }
            }
        }
    }

    protected function _hasCredentials($marketCode)
    {
        foreach ($this->_credentialFields as $field) {
            if ('' == $this->getValue($field)) {
                return FALSE;
            }
        }

        if ('' == $this->getValue("{$marketCode}_affiliate_id")) {
            return FALSE;
        }

        return TRUE;
    }

    //
    // Getters & setters
    //

    public function getWordingFields()
    {
        return $this->_wordingFields;
    }

    public function getMarkets()
    {
        return $this->_markets;
    }

    public function getMarketCodes()
    {
        return array_keys($this->_markets);
    }
}